#include<stdio.h>
#include<stdlib.h>
#include<qstring.h>
#include<qdict.h>
#include<qptrlist.h>
#include<qfileinfo.h>

#include<unistd.h>
#include<fcntl.h>
#include<signal.h>
#include<sys/utsname.h>

class Conf;

class qc_obj
{
public:
	qc_obj(Conf *c);
	virtual ~qc_obj() {}

	virtual bool required() const=0;
	virtual QString name() const=0;
	virtual bool exec()=0;

	Conf *conf;
};

class Conf
{
public:
	Conf() : vars(17)
	{
		list.setAutoDelete(true);
		vars.setAutoDelete(true);

		vars.insert("QMAKE_INCDIR_X11", new QString(X11_INC));
		vars.insert("QMAKE_LIBDIR_X11", new QString(X11_LIBDIR));
		vars.insert("QMAKE_LIBS_X11",   new QString(X11_LIB));
		vars.insert("QMAKE_CC",         new QString(CC));

		//QDictIterator<QString> it(vars);
		//for(QString *s; (s = it.current()); ++it) {
		//	printf("%s: %s\n", it.currentKey().latin1(), s->latin1());
		//}
		//printf("------------\n\n");
	}

	~Conf()
	{
	}

	void added(qc_obj *o)
	{
		list.append(o);
	}

	bool exec()
	{
		QPtrListIterator<qc_obj> it(list);
		for(qc_obj *o; (o = it.current()); ++it) {
			printf("Checking for %s ...", o->name().latin1());
			fflush(stdout);
			bool ok = o->exec();
			if(ok)
				printf(" yes\n");
			else
				printf(" no\n");
			if(!ok && o->required()) {
				printf("\nError: need %s!\n\n", o->name().latin1());
				return false;
			}
		}
		return true;
	}

	const QString & qvar(const QString &s)
	{
		QString *p = vars.find(s);
		if(p)
			return *p;
		else
			return blank;
	}

	QString expandIncludes(const QString &inc)
	{
		return QString("-I") + inc;
	}

	QString expandLibs(const QString &lib)
	{
		return QString("-L") + lib;
	}

	int doCommand(const QString &s)
	{
		//printf("[%s]\n", s.latin1());
		int r = system((s + " 1>/dev/null 2>/dev/null").latin1());
		//printf("returned: %d\n", r);
		return r;
	}

	bool findLibrary(const QString &h, const QString &lib)
	{
		QFileInfo fi;
		bool ok;
		char *atest =
			"int main()\n"
			"{\n"
			"    return 0;\n"
			"}\n";

		// find header
		ok = false;
		fi.setFile("/usr/include/" + h);
		if(fi.exists())
			ok = true;
		else {
			fi.setFile("/usr/local/include/" + h);
			if(fi.exists()) {
				INCLUDEPATH += " /usr/local/include";
				ok = true;
			}
		}
		if(!ok)
			return false;

		FILE *f;
		f = fopen("atest.c", "w");
		if(!f)
			return 0;
		fwrite(atest, strlen(atest), 1, f);
		fclose(f);

		QString str = qvar("QMAKE_CC") + " atest.c -o atest -l" + lib;
		int r = doCommand(str);
		remove("atest.c");
		remove("atest");
		if(r == 0)
			return true;

		return false;
	};

	QString DEFINES;
	QString INCLUDEPATH;
	QString LIBS;
	QString extra;

private:
	QPtrList<qc_obj> list;
	QDict<QString> vars;
	QString blank;
};

qc_obj::qc_obj(Conf *c)
{
	conf = c;
	conf->added(this);
}


//----------------------------------------------------------------------------
// qc_xss
//----------------------------------------------------------------------------
class qc_xss : public qc_obj
{
public:
	enum { OK, NEEDLIB, FAIL };
	qc_xss(Conf *c) : qc_obj(c)
	{
	}

	~qc_xss()
	{
		remove("xssprobe_test.c");
		remove("xssprobe_test.o");
		remove("xssprobe_test");
	}

	bool required() const
	{
		return false;
	}

	QString name() const
	{
		return "the XScreenSaver extension";
	}

	int do_write()
	{
		char *xsstest =
			"#include<X11/Xlib.h>\n"
			"#include<X11/Xutil.h>\n"
			"#include<X11/extensions/scrnsaver.h>\n"
			"\n"
			"int main()\n"
			"{\n"
			"    XScreenSaverQueryExtension(NULL, NULL, NULL);\n"
			"    return 0;\n"
			"}\n";

		FILE *f;
		f = fopen("xssprobe_test.c", "w");
		if(!f)
			return 0;
		fwrite(xsstest, strlen(xsstest), 1, f);
		fclose(f);

		return 1;
	}

	int do_compile()
	{
		QString inc = conf->expandIncludes(conf->qvar("QMAKE_INCDIR_X11"));
		QString str = conf->qvar("QMAKE_CC") + " -c " + inc + " xssprobe_test.c -o xssprobe_test.o";
		int r = conf->doCommand(str);
		if(r == 0)
			return 1;
		else
			return 0;
	}

	int do_link()
	{
		QString lib = conf->expandLibs(conf->qvar("QMAKE_LIBDIR_X11"));
		QString inc = conf->expandIncludes(conf->qvar("QMAKE_INCDIR_X11"));
		QString str = conf->qvar("QMAKE_CC") + " xssprobe_test.o -o xssprobe_test " + lib + ' ' + conf->qvar("QMAKE_LIBS_X11");
		int r = conf->doCommand(str);
		if(r == 0)
			return 1;
		else
			return 0;
	}

	int do_linkLib()
	{
		QString lib = conf->expandLibs(conf->qvar("QMAKE_LIBDIR_X11"));
		QString inc = conf->expandIncludes(conf->qvar("QMAKE_INCDIR_X11"));
		QString str = conf->qvar("QMAKE_CC") + " xssprobe_test.o -o xssprobe_test " + lib + ' ' + conf->qvar("QMAKE_LIBS_X11") + " -lXss";
		int r = conf->doCommand(str);
		if(r == 0)
			return 1;
		else
			return 0;
	}

	int do_all()
	{
		if(!do_write())
			return FAIL;
		if(!do_compile())
			return FAIL;
		if(do_link())
			return OK;
		if(do_linkLib())
			return NEEDLIB;
		return FAIL;
	}

	bool exec()
	{
		int r = do_all();
		if(r == OK)
			return true;
		if(r == NEEDLIB) {
			conf->LIBS += " -lXss";
			return true;
		}
		else {
			conf->DEFINES += "NO_XSS";
			return false;
		}
	}
};


//----------------------------------------------------------------------------
// qc_zlib
//----------------------------------------------------------------------------
class qc_zlib : public qc_obj
{
public:
	qc_zlib(Conf *c) : qc_obj(c) {}
	bool required() const { return true; }
	QString name() const { return "zlib"; }
	bool exec()
	{
		if(!conf->findLibrary("zlib.h", "z"))
			return false;

		conf->LIBS += " -lz";
		return true;
	}
};


//----------------------------------------------------------------------------
// qc_qt
//----------------------------------------------------------------------------
class qc_qt : public qc_obj
{
public:
	qc_qt(Conf *c) : qc_obj(c) {}
	bool required() const { return true; }
	QString name() const { return "Qt (Multithreaded) >= 3.1"; }
	bool exec()
	{
		if(QT_VERSION >= 0x030100 && QT_THREAD_SUPPORT)
			return true;
		else
			return false;
	}
};


//----------------------------------------------------------------------------
// qc_dnotify
//----------------------------------------------------------------------------
class qc_dnotify : public qc_obj
{
public:
	qc_dnotify(Conf *c) : qc_obj(c)
	{
	}

	~qc_dnotify()
	{
		remove("ftest.c");
		remove("ftest.o");
	}

	bool required() const { return false; }
	QString name() const { return "Linux DNOTIFY"; }

	bool do_write()
	{
		char *fdata =
			"#define _GNU_SOURCE\n"
			"#include<unistd.h>\n"
			"#include<fcntl.h>\n"
			"#include<signal.h>\n"
			"#include<sys/utsname.h>\n"
			"\n"
			"int main()\n"
			"{\n"
			"    DN_DELETE|DN_CREATE|DN_RENAME|DN_MULTISHOT|DN_MODIFY|DN_ATTRIB;\n"
			"    return 0;\n"
			"}\n";

		FILE *f;
		f = fopen("ftest.c", "w");
		if(!f)
			return false;
		fwrite(fdata, strlen(fdata), 1, f);
		fclose(f);

		return true;
	}

	bool do_compile()
	{
		QString str = conf->qvar("QMAKE_CC") + " -c ftest.c -o ftest.o";
		int r = conf->doCommand(str);
		if(r == 0)
			return true;
		else
			return false;
	}

	bool exec()
	{
		bool supports_dnotify = true; // not guilty until proven guilty
		struct utsname uts;
		int major, minor, patch;
		if(uname(&uts) < 0)
			supports_dnotify = false; // *shrug*
		else if(sscanf(uts.release, "%d.%d.%d", &major, &minor, &patch) != 3)
			supports_dnotify = false; // *shrug*
		else if( major * 1000000 + minor * 1000 + patch < 2004019 ) // <2.4.19
			supports_dnotify = false;
		if(!supports_dnotify)
			return false;

		if(!do_write())
			return false;
		if(!do_compile())
			return false;

		conf->DEFINES += "HAVE_DNOTIFY";
		return true;
	}
};


//----------------------------------------------------------------------------
// qc_kde
//----------------------------------------------------------------------------
class qc_kde : public qc_obj
{
public:
	qc_kde(Conf *c) : qc_obj(c) {}
	bool required() const { return false; }
	QString name() const { return "KDE"; }
	bool exec()
	{
		char *p = getenv("KDEDIR");
		if(!p)
			return false;

		conf->extra += QString("KDE = %1").arg(p);
		return true;
	}
};


//----------------------------------------------------------------------------
// main
//----------------------------------------------------------------------------
int main()
{
	Conf *conf = new Conf;
	new qc_qt(conf);   // Qt 3.1
	new qc_zlib(conf); // zlib
	new qc_xss(conf);  // XScreenSaver
	new qc_dnotify(conf);  // Linux DNOTIFY
	new qc_kde(conf);  // KDE

	if(conf->exec()) {
		FILE *f = fopen("qconf.pri", "w");
		if(!f) {
			printf("Error writing qconf.pri\n");
			return 0;
		}
		fprintf(f, "# qconf\n");
		if(!conf->DEFINES.isEmpty())
			fprintf(f, "DEFINES += %s\n", conf->DEFINES.latin1());
		if(!conf->INCLUDEPATH.isEmpty())
			fprintf(f, "INCLUDEPATH += %s\n", conf->INCLUDEPATH.latin1());
		if(!conf->LIBS.isEmpty())
			fprintf(f, "LIBS    += %s\n", conf->LIBS.latin1());
		if(!conf->extra.isEmpty())
			fprintf(f, "%s", conf->extra.latin1());

		fclose(f);
	}

	delete conf;

	return 0;
}
