/****************************************************************************
** iconset.h - various graphics handling classes
** Copyright (C) 2001, 2002  Justin Karneges
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
**
****************************************************************************/

#ifndef ICONSET_H
#define ICONSET_H

#include<qpixmap.h>
#include<qimage.h>
#include<qptrvector.h>
#include<qvaluelist.h>
#include<qmime.h>
#include<qregexp.h>
#include<qpushbutton.h>
#include"xmpp_types.h"

class PsiEvent;
class QTimer;

class Impix
{
public:
	Impix();
	Impix(const QPixmap &);
	Impix(const QImage &);
	~Impix();
	Impix(const Impix &);
	Impix & operator=(const Impix &from);

	bool isNull() const;

	const QPixmap & pixmap() const;
	const QImage & image() const;

	bool load(const QString &, const QStringList &);
	void unload();

	void setPixmap(const QPixmap &);
	void setImage(const QImage &);

	operator const QPixmap &() const { return pixmap(); }
	operator const QImage &() const { return image(); }

private:
	QPixmap *v_pixmap;
	QImage *v_image;
};


class Anim
{
public:
	Anim();
	~Anim();

	Impix & frame(int) const;
	Impix & base() const;
	bool isAnim() { return v_isAnim; }
	int numFrames() { return v_numFrames; }

	bool load(const QString &, const QStringList &);
	void unload();

private:
	QPtrVector<Impix> seq;
	int v_numFrames;
	bool v_isAnim;
};

class Emoticon : public Impix
{
public:
	Emoticon(const QString &name="", const QRegExp &reg=QRegExp());

	const QString &name() const;
	const QRegExp &regExp() const;
	void setName(const QString &);
	void setRegExp(const QRegExp &);

private:
	QString v_name;
	QRegExp v_reg;
};

class EmoticonList : public QValueList<Emoticon>
{
public:
	EmoticonList();

	bool add(const QStringList &dirs, const QString &name, const QRegExp &reg);
	EmoticonList::Iterator find(const QString &);
	QMimeSourceFactory *generateFactory() const;
	void addToFactory(QMimeSourceFactory *) const;
};

// WARNING WARNING, some classes like EventDlg depend on members in this class not moving (since they are directly referred to).  fix someday..
class PsiIconSet
{
public:
	PsiIconSet();

	const Impix & status(const Jabber::Status &) const;
	const Impix & status(int) const;
	Anim & event2anim(PsiEvent *);

	bool load(const QString &);
	void unload();
	static void getIconSetsList(QStringList &names, QStringList &descriptions);

	Anim message, system, chat, headline, conn, needkey;
	Impix online, offline, away, xa, dnd, invisible, noauth, perr, arrow[3], add, remove, send, history, info, url, profile, groupchat, ask;
	Impix psimain, ssl_yes, ssl_no, chatclear, account;
	Impix options, play_sounds, quit, help, xml;
	Impix arrow_left, arrow_right, arrow_up, arrow_down;
	Impix browse, registerAgent, search, getTime, getVersion, getLastTime, refresh, stop;

	EmoticonList emoticons;

private:
	bool loadSet(const QString &);
};

class AnimatedButton : public QPushButton
{
	Q_OBJECT
public:
	AnimatedButton(QWidget *parent=0, const char *name=0, bool animOnly=false);

	void setAnim(Anim *);
	bool isActive() { return v_isActive; }

private:
	Anim *anim;
	bool v_isActive, animOnly;
	QTimer *t;
	int animStep;

	void changeIcon(const QPixmap &);

private slots:
	void animate();
};

#endif
